#region Using Statements
using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
#endregion

namespace FallingBlockPuzzle {

	public abstract class BlockVisual : DrawableGameComponent {
		public const int Width = 16;
		public const int Height = 16;

		protected Texture2D texture;
		protected SpriteBatch sprites;
		protected readonly Palette palette = new Palette();
		readonly ContentManager content;

		public BlockVisual(Game game, ContentManager content)
			: base(game) {
			this.content = content;
		}
		protected override void LoadGraphicsContent(bool loadAllContent) {
			if (loadAllContent) {
				sprites = new SpriteBatch(GraphicsDevice);
				texture = content.Load<Texture2D>("content/texture/Orb");
			}
			base.LoadGraphicsContent(loadAllContent);
		}
	}
	public class BlocksVisual : BlockVisual {
		readonly Blocks blocks;
		readonly PieceControl control;
		Texture2D backgroundTexture;
		public BlocksVisual(Game game, ContentManager content, Blocks blocks, PieceControl control)
			: base(game, content) {
			this.blocks = blocks;
			this.control = control;
		}
		protected override void LoadGraphicsContent(bool loadAllContent) {
			if (loadAllContent) {
				backgroundTexture = new Texture2D(GraphicsDevice, 1, 1, 1, ResourceUsage.None, SurfaceFormat.Color);
				backgroundTexture.SetData(new Color[] { Color.White });

			}
			base.LoadGraphicsContent(loadAllContent);
		}
		public override void Draw(GameTime gameTime) {
			sprites.Begin();

			sprites.Draw(backgroundTexture, new Rectangle(0, 0, blocks.Width * BlockVisual.Width, blocks.Height * BlockVisual.Height),
				control.Overflow ? Color.Orange : Color.White);


			Point position;
			for (position.Y = 0; position.Y < blocks.Height; position.Y++) {
				for (position.X = 0; position.X < blocks.Width; position.X++) {
					sprites.Draw(texture,
						new Rectangle(position.X * BlockVisual.Width, position.Y * BlockVisual.Height,
						BlockVisual.Width, BlockVisual.Height), palette[blocks[position].Type]);
				}
			}
			sprites.End();
			base.Draw(gameTime);
		}
	}
	public class PieceVisual : BlockVisual {
		readonly Piece piece;

		public PieceVisual(Game game, ContentManager content, Piece piece)
			: base(game, content) {
			this.piece = piece;
		}
		public override void Draw(GameTime gameTime) {
			if (piece.Enabled) {
				sprites.Begin();
				Point position;
				for (position.Y = 0; position.Y < piece.Height; position.Y++) {
					for (position.X = 0; position.X < piece.Width; position.X++) {
						Block block = piece[position];
						sprites.Draw(texture, block.Offset, new Rectangle(0, 0, BlockVisual.Width, BlockVisual.Height),
							palette[block.Type]);
					}
				}
				sprites.End();
			}
			base.Draw(gameTime);
		}
	}
	public class PiecesVisual : BlockVisual {
		readonly List<Piece> pieces;

		public PiecesVisual(Game game, ContentManager content, List<Piece> pieces)
			: base(game, content) {
			this.pieces = pieces;
		}
		public override void Draw(GameTime gameTime) {
			sprites.Begin();

			Point position;

			for (int i = 0; i < pieces.Count; i++) {
				Piece piece = pieces[i];
				if (!piece.Enabled) continue;
				for (position.Y = 0; position.Y < piece.Height; position.Y++) {
					for (position.X = 0; position.X < piece.Width; position.X++) {
						Block block = piece[position];
						sprites.Draw(texture, block.Offset, new Rectangle(0, 0, BlockVisual.Width, BlockVisual.Height),
							palette[block.Type]);
					}
				}
			}
			sprites.End();
			base.Draw(gameTime);
		}
	}

}
